@@ -33,6 +33,10 @@ def is_impala(engine: sa.engine.Engine) -> bool:
     return engine.name == "impala"
 
 
+def is_db2(engine: sa.engine.Engine) -> bool:
+    return engine.name == "ibm_db_sa"
+
+
 def get_table_columns(table, column_names):
     return [table.c[column_name] for column_name in column_names]
 
@@ -421,6 +425,15 @@ def get_date_span(engine, ref, date_column_name):
                 )
             ]
         )
+    elif is_db2(engine):
+        selection = sa.select(
+            [
+                sa.func.days_between(
+                    sa.func.max(column),
+                    sa.func.min(column),
+                )
+            ]
+        )
     else:
         raise NotImplementedError(
             "Date spans not yet implemented for this sql dialect."
@@ -663,6 +676,14 @@ def get_date_gaps(
             )
             > legitimate_gap_size
         )
+    elif is_db2(engine):
+        gap_condition = (
+            sa.func.days_between(
+                start_table.c[start_column],
+                end_table.c[end_column],
+            )
+            > legitimate_gap_size
+        )
     else:
         raise NotImplementedError(f"Date gaps not yet implemented for {engine.name}.")
 
@@ -756,6 +777,39 @@ def get_mean(engine, ref):
     return get_column(engine, ref, aggregate_operator=column_operator)
 
 
+def get_percentile(engine, ref, percentage):
+    row_count = "dj_row_count"
+    row_num = "dj_row_num"
+    column_name = ref.get_column(engine)
+    column = ref.get_selection(engine).subquery().c[column_name]
+    subquery = (
+        sa.select(
+            [
+                column,
+                sa.func.row_number().over(order_by=column).label(row_num),
+                sa.func.count().over(partition_by=None).label(row_count),
+            ]
+        )
+        .where(column.is_not(None))
+        .subquery()
+    )
+
+    constrained_selection = (
+        sa.select(subquery.columns)
+        .where(subquery.c[row_num] * 100.0 / subquery.c[row_count] <= percentage)
+        .subquery()
+    )
+
+    max_selection = sa.select(
+        sa.func.max(constrained_selection.c[row_num])
+    ).scalar_subquery()
+    selection = sa.select(constrained_selection.c[column_name]).where(
+        constrained_selection.c[row_num] == max_selection
+    )
+    result = engine.connect().execute(selection).scalar()
+    return result, [selection]
+
+
 def get_min_length(engine, ref):
     def column_operator(column):
         return sa.func.min(sa.func.length(column))
@@ -835,16 +889,22 @@ def get_unique_count_union(engine, ref, ref2):
     return result, [selection]
 
 
-def contains_null(engine, ref):
-    selection = ref.get_selection(engine)
-    subquery = selection.distinct().alias()
-    selection = (
+def get_missing_fraction(engine, ref):
+    selection = ref.get_selection(engine).subquery()
+    n_rows_total_selection = sa.select([sa.func.count()]).select_from(selection)
+    n_rows_missing_selection = (
         sa.select([sa.func.count()])
-        .select_from(subquery)
-        .where(subquery.c[ref.get_column(engine)].is_(None))
+        .select_from(selection)
+        .where(selection.c[ref.get_column(engine)].is_(None))
+    )
+    with engine.connect() as connection:
+        n_rows_total = connection.execute(n_rows_total_selection).scalar()
+        n_rows_missing = connection.execute(n_rows_missing_selection).scalar()
+
+    return (
+        n_rows_missing / n_rows_total,
+        [n_rows_total_selection, n_rows_missing_selection],
     )
-    n_rows = engine.connect().execute(selection).scalar()
-    return n_rows > 0, [selection]
 
 
 def get_column_names(engine, ref):
