@@ -124,7 +124,10 @@ class WithinRequirement(Requirement):
     def add_primary_key_definition_constraint(
         self, primary_keys: List[str], name: str = None
     ):
-        """Primary keys of exactly equal to given column names in the database."""
+        """Check that the primary key constraints in the database are exactly equal to the given column names.
+
+        Note that this doesn't actually check that the primary key values are unique across the table.
+        """
         ref = DataReference(self.data_source)
         self._constraints.append(
             miscs_constraints.PrimaryKeyDefinition(ref, primary_keys, name=name)
@@ -144,7 +147,7 @@ class WithinRequirement(Requirement):
         Given a set of columns, satisfy conditions of a primary key, i.e.
         uniqueness of tuples from said columns. This constraint has a tolerance
         for inconsistencies, expressed via max_duplicate_fraction. The latter
-        suggests that the number of uniques from said colums is larger or equal
+        suggests that the number of uniques from said columns is larger or equal
         to (1 - max_duplicate_fraction) the number of rows.
 
         If infer_pk_columns is True, columns will be retrieved from the primary keys.
@@ -174,7 +177,27 @@ class WithinRequirement(Requirement):
         self, column: str, condition: Condition = None, name: str = None
     ):
         ref = DataReference(self.data_source, [column], condition)
-        self._constraints.append(miscs_constraints.NullAbsence(ref, name=name))
+        self._constraints.append(
+            miscs_constraints.MaxNullFraction(ref, max_null_fraction=0, name=name)
+        )
+
+    def add_max_null_fraction_constraint(
+        self,
+        column: str,
+        max_null_fraction: float,
+        condition: Condition = None,
+        name: str = None,
+    ):
+        """Assert that ``column`` has less than a certain fraction of ``NULL`` values.
+
+        ``max_null_fraction`` is expected to lie within [0, 1].
+        """
+        ref = DataReference(self.data_source, [column], condition)
+        self._constraints.append(
+            miscs_constraints.MaxNullFraction(
+                ref, max_null_fraction=max_null_fraction, name=name
+            )
+        )
 
     def add_n_rows_equality_constraint(
         self, n_rows: int, condition: Condition = None, name: str = None
@@ -384,6 +407,43 @@ class WithinRequirement(Requirement):
             )
         )
 
+    def add_numeric_percentile_constraint(
+        self,
+        column: str,
+        percentage: float,
+        expected_percentile: float,
+        max_absolute_deviation: Optional[float] = None,
+        max_relative_deviation: Optional[float] = None,
+        condition: Condition = None,
+        name: str = None,
+    ):
+        """Assert that the ``percentage``-th percentile is approximately ``expected_percentile``.
+
+        The percentile is defined as the value present in ``column`` for which
+        ``percentage`` % of the values in ``column`` are less or equal. ``NULL`` values
+        are ignored.
+
+        Hence, if ``percentage`` is less than the inverse of the number of non-``NULL`` rows,
+        ``None`` is received as the ``percentage`` -th percentile.
+
+        ``percentage`` is expected to be provided in percent. The median, for example, would
+        correspond to ``percentage=50``.
+
+        At least one of ``max_absolute_deviation`` and ``max_relative_deviation`` must
+        be provided.
+        """
+        ref = DataReference(self.data_source, [column], condition)
+        self._constraints.append(
+            numeric_constraints.NumericPercentile(
+                ref,
+                percentage=percentage,
+                expected_percentile=expected_percentile,
+                max_absolute_deviation=max_absolute_deviation,
+                max_relative_deviation=max_relative_deviation,
+                name=name,
+            )
+        )
+
     def add_date_min_constraint(
         self,
         column: str,
@@ -660,7 +720,7 @@ class WithinRequirement(Requirement):
         n_counterexamples: int = 5,
     ):
         """
-        Assesses whether the values in a column match a given regular expresion pattern.
+        Assesses whether the values in a column match a given regular expression pattern.
 
         The option ``allow_none`` can be used in cases where the column is defined as
         nullable and contains null values.
@@ -702,7 +762,7 @@ class WithinRequirement(Requirement):
         n_counterexamples: int = 5,
     ):
         """
-        Assesses whether the values in a column match a given regular expresion pattern.
+        Assesses whether the values in a column match a given regular expression pattern.
 
         How the tolerance factor is calculated can be controlled with the ``aggregated``
         flag. When ``True``, the tolerance is calculated using unique values. If not, the
@@ -715,7 +775,7 @@ class WithinRequirement(Requirement):
         When using this method, the regex matching will take place in database, which is
         only supported for Postgres, Sqllite and Snowflake. Note that for this
         feature is only for Snowflake when using sqlalchemy-snowflake >= 1.4.0. As an
-        altenative, ``add_varchar_regex_constraint`` performs the regex matching in memory.
+        alternative, ``add_varchar_regex_constraint`` performs the regex matching in memory.
         This is typically slower and more expensive in terms of memory but available
         on all supported database mamangement systems.
         """
@@ -764,7 +824,7 @@ class WithinRequirement(Requirement):
         condition: Condition = None,
         name: str = None,
     ):
-        """Chek whether array aggregate corresponds to an integer range.
+        """Check whether array aggregate corresponds to an integer range.
 
         The ``DataSource`` is grouped by ``columns``. Sql's ``array_agg`` function is then
         applied to the ``aggregate_column``.
@@ -873,12 +933,12 @@ class BetweenRequirement(Requirement):
         date_column: Optional[str] = None,
         date_column2: Optional[str] = None,
     ):
-        """Create a ``BetwenTableRequirement`` based on sqlalchemy expressions.
+        """Create a ``BetweenTableRequirement`` based on sqlalchemy expressions.
 
         Any sqlalchemy object implementing the ``alias`` method can be passed as an
         argument for the ``expression1`` and ``expression2`` parameters. This could,
         e.g. be a ``sqlalchemy.Table`` object or the result of a ``sqlalchemy.select``
-        invokation.
+        invocation.
 
         ``name1`` and ``name2`` will be used to represent the expressions in error messages,
         respectively.
@@ -1065,6 +1125,31 @@ class BetweenRequirement(Requirement):
             )
         )
 
+    def add_max_null_fraction_constraint(
+        self,
+        column1: str,
+        column2: str,
+        max_relative_deviation: float,
+        condition1: Condition = None,
+        condition2: Condition = None,
+        name: str = None,
+    ):
+        """Assert that the fraction of ``NULL`` values of one is at most that of the other.
+
+        Given that ``column2``\'s underlying data has a fraction ``q`` of ``NULL`` values, the
+        ``max_relative_deviation`` parameter allows ``column1``\'s underlying data to have a
+        fraction ``(1 + max_relative_deviation) * q`` of ``NULL`` values.
+        """
+        ref = DataReference(self.data_source, [column1], condition1)
+        ref2 = DataReference(self.data_source2, [column2], condition2)
+        self._constraints.append(
+            miscs_constraints.MaxNullFraction(
+                ref,
+                ref2=ref2,
+                max_relative_deviation=max_relative_deviation,
+            )
+        )
+
     def add_numeric_min_constraint(
         self,
         column1: str,
@@ -1230,6 +1315,45 @@ class BetweenRequirement(Requirement):
             )
         )
 
+    def add_numeric_percentile_constraint(
+        self,
+        column1: str,
+        column2: str,
+        percentage: float,
+        max_absolute_deviation: Optional[float] = None,
+        max_relative_deviation: Optional[float] = None,
+        condition1: Optional[Condition] = None,
+        condition2: Optional[Condition] = None,
+        name: str = None,
+    ):
+        """Assert that the ``percentage``-th percentile is approximately equal.
+
+        The percentile is defined as the value present in ``column1`` / ``column2``
+        for which ``percentage`` % of the values in ``column1`` / ``column2`` are
+        less or equal. ``NULL`` values are ignored.
+
+        Hence, if ``percentage`` is less than the inverse of the number of non-``NULL``
+        rows, ``None`` is received as the ``percentage``-th percentile.
+
+        ``percentage`` is expected to be provided in percent. The median, for example,
+        would correspond to ``percentage=50``.
+
+        At least one of ``max_absolute_deviation`` and ``max_relative_deviation`` must
+        be provided.
+        """
+        ref = DataReference(self.data_source, [column1], condition1)
+        ref2 = DataReference(self.data_source2, [column2], condition2)
+        self._constraints.append(
+            numeric_constraints.NumericPercentile(
+                ref,
+                percentage=percentage,
+                max_absolute_deviation=max_absolute_deviation,
+                max_relative_deviation=max_relative_deviation,
+                ref2=ref2,
+                name=name,
+            )
+        )
+
     def add_date_min_constraint(
         self,
         column1: str,
@@ -1383,7 +1507,7 @@ class BetweenRequirement(Requirement):
         Rows from T1 are indexed in columns1, rows from T2 are indexed in ``columns2``.
 
         In particular, the operation ``|T1-T2|`` relies on a sql ``EXCEPT`` statement. In
-        constrast to ``EXCEPT ALL``, this should lead to a set subtraction instead of
+        contrast to ``EXCEPT ALL``, this should lead to a set subtraction instead of
         a multiset subtraction. In other words, duplicates in T1 are treated as
         single occurrences.
         """
